﻿using Syncfusion.SfChart.XForms;
using System;
using System.Collections.ObjectModel;
using Xamarin.Forms;
using System.Collections.Generic;

namespace LiveChartSample
{
    public class ChartPage : ContentPage
    {
		public ObservableCollection<Model> Data { get; set; }
        Random random;
        NumericalAxis xAxis;
        int i;
		double visibleStart;
		double visibleEnd;

        public ChartPage()
        {
            Data = new ObservableCollection<Model>();
            random = new Random();

            StackLayout layout = new StackLayout();

            SfChart chart = new SfChart();
            chart.VerticalOptions = LayoutOptions.FillAndExpand;
            chart.HorizontalOptions = LayoutOptions.FillAndExpand;

            xAxis = new NumericalAxis();
            chart.PrimaryAxis = xAxis;

            NumericalAxis yAxis = new NumericalAxis();
            yAxis.Minimum = 10;
            yAxis.Maximum = 50;
            chart.SecondaryAxis = yAxis;

            FastLineSeries lineSeries = new FastLineSeries();
            lineSeries.ItemsSource = GenerateDataPoints();
            lineSeries.XBindingPath = "XValue";
            lineSeries.YBindingPath = "YValue";
            chart.Series.Add(lineSeries);

            ChartZoomPanBehavior zoomPanBehavior = new ChartZoomPanBehavior();
            zoomPanBehavior.EnableDoubleTap = false;
            zoomPanBehavior.EnableSelectionZooming = false;
            zoomPanBehavior.ZoomMode = ZoomMode.X;
            chart.ChartBehaviors.Add(zoomPanBehavior);

            Button button = new Button();
            button.Text = "LoadData";
            button.Clicked += Button_Clicked;

            chart.ZoomEnd += Chart_ZoomEnd;
			chart.Scroll += Chart_Scroll;
            layout.Children.Add(chart);
            layout.Children.Add(button);
            Content = layout;
        }

        private void Chart_ZoomEnd(object sender, ChartZoomEventArgs e)
        {
			GetVisibleRangeValues();
		}

		void Chart_Scroll(object sender, ChartScrollEventArgs e)
		{
			GetVisibleRangeValues();
		}

		private void GetVisibleRangeValues()
		{
			List<double> visibleValues = DependencyService.Get<IAxisDependencyService>().GetVisibleRange(xAxis);
			visibleStart = visibleValues[0];
			visibleEnd = visibleValues[1];
		}

		private void SetCurrentZoomPosition()
		{
			var start = (double)Data[0].XValue;
			var end = (double)Data[Data.Count - 1].XValue;
			double delta = GetDelta(start, end);

			xAxis.ZoomPosition = (visibleStart - start) / delta;
			xAxis.ZoomFactor = (visibleEnd - visibleStart) / delta;
		}

        private void Button_Clicked(object sender, EventArgs e)
        {
            LoadData();
			SetCurrentZoomPosition();
		}

        double GetDelta(double minimum, double maximum)
        {
            return (minimum > maximum) ? minimum - maximum : maximum - minimum;
        }

        public ObservableCollection<Model> GenerateDataPoints()
        {
            for (i = 1; i <= 10; i++)
            {
                Data.Add(new Model(i, random.Next(10, 50)));
            }
			i--;
            return Data;
        }

        private bool LoadData()
        {
            i = i + 1;
            Data.Add(new Model(i, random.Next(10,50)));
            return true;
        }
    }

    public class Model
    {
        public double XValue { get; set; }

        public double YValue { get; set; }

        public Model(double _xVal, double _yVal)
        {
            XValue = _xVal;
            YValue = _yVal;
        }
    }


	public interface IAxisDependencyService
	{
		List<double> GetVisibleRange(NumericalAxis axis);
	}
}







